# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solumclient.common.apiclient import client
from solumclient.common.apiclient import fake_client
from solumclient.tests import base
from solumclient.v1 import plan


plan_list = [
    {
        'name': 'Example plan 1',
        'artifacts': (
            [{'name': 'My python app',
              'artifact_type': 'git_pull',
              'content': {'href': 'git://example.com/project.git'},
              'requirements': [{
                  'requirement_type': 'git_pull',
                  'language_pack': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                  'fulfillment': '1dae5a09ef2b4d8cbf3594b0eb4f6b94'}]}]),
        'services': [{'name': 'Build Service',
                      'id': 'build',
                      'characteristics': ['python_build_service']}],
        'description': 'A plan with no services or artifacts shown'
    },
    {
        'name': 'Example plan 2',
        'artifacts': (
            [{'name': 'My java app',
              'artifact_type': 'git_pull',
              'content': {'href': 'git://example.com/project.git'},
              'requirements': [{
                  'requirement_type': 'git_pull',
                  'language_pack': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                  'fulfillment': '1dae5a09ef2b4d8cbf3594b0eb4f6b94'}]}]),
        'services': [{'name': 'Build Service',
                      'id': 'build',
                      'characteristics': ['python_build_service']}],
        'description': 'A plan with no services or artifacts shown'
    },
]

artifacts = [{'name': 'My python app',
              'artifact_type': 'git_pull',
              'content': {'href': 'git://example.com/project.git'},
              'requirements': [{
                  'requirement_type': 'git_pull',
                  'language_pack': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                  'fulfillment': '1dae5a09ef2b4d8cbf3594b0eb4f6b94'}]}]

services = [{'name': 'Build Service',
             'id': 'build',
             'characteristics': ['python_build_service']}]

plan_fixture = {
    'uri': 'http://example.com/v1/plans/p1',
    'name': 'Example plan',
    'type': 'plan',
    'tags': ['small'],
    'artifacts': artifacts,
    'services': services,
    'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
    'user_id': '55f41cf46df74320b9486a35f5d28a11',
    'description': 'A plan with no services or artifacts shown'
}

fixtures_list = {
    '/v1/plans': {
        'GET': (
            {},
            plan_list
        ),
    }
}

fixtures_list_empty = {
    '/v1/plans': {
        'GET': (
            {},
            []
        ),
    }
}


fixtures_get = {
    '/v1/plans/p1': {
        'GET': (
            {},
            plan_fixture
        ),
    }
}


fixtures_create = {
    '/v1/plans': {
        'POST': (
            {},
            plan_fixture
        ),
    }
}

fixtures_put = {
    '/v1/plans/p1': {
        'PUT': (
            {},
            plan_fixture
        ),
    }
}


class PlanManagerTest(base.TestCase):

    def assert_plan_obj(self, plan_obj):
        self.assertIn('Plan', repr(plan_obj))
        self.assertIn('Artifact', repr(plan_obj.artifacts[0]))
        self.assertIn('ServiceReference', repr(plan_obj.services[0]))
        self.assertEqual(plan_fixture['uri'], plan_obj.uri)
        self.assertEqual(plan_fixture['type'], plan_obj.type)
        self.assertEqual(plan_fixture['project_id'], plan_obj.project_id)
        self.assertEqual(plan_fixture['user_id'], plan_obj.user_id)

    def test_list_all(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_list)
        api_client = client.BaseClient(fake_http_client)
        plan.PlanManager(api_client)
        # NOTE(stannie): will re-enable this test once
        # https://bugs.launchpad.net/solum/+bug/1331093 is committed.
        # FakeHTTPClient doesn't manage YAML properly but since this method
        # will use the json content-type once implemented in the API, this can
        # stay temporary disabled.

    def test_list_empty(self):
        fake_http_client = fake_client.FakeHTTPClient(
            fixtures=fixtures_list_empty)
        api_client = client.BaseClient(fake_http_client)
        mgr = plan.PlanManager(api_client)
        self.assertEqual([], mgr.list())

    def test_create(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_create)
        api_client = client.BaseClient(fake_http_client)
        mgr = plan.PlanManager(api_client)
        plan_obj = mgr.create('version: 1\nname: ex_plan1\ndescription: dsc1.')
        self.assert_plan_obj(plan_obj)

    def test_plan_create_post_failure(self):
        api_client = mock.MagicMock()
        api_client.post.side_effect = Exception("Bad data")
        try:
            mgr = plan.PlanManager(api_client)
            mgr.create('version: 1\nname: ex_plan1\ndescription: dsc1.')
        except Exception:
            self.assertTrue(True)

    def test_plan_create_post_success(self):
        api_client = mock.MagicMock()
        dummy_data = 'version: 1\nname: ex_plan1\ndescription: dsc1.'
        response = mock.MagicMock()
        setattr(response, 'content', dummy_data)
        api_client.post.return_value = response
        try:
            mgr = plan.PlanManager(api_client)
            plan_obj = mgr.create(dummy_data)
            assert plan_obj is not None
            assert plan_obj.name == 'ex_plan1'
            assert plan_obj.description == 'dsc1.'
            assert plan_obj.version == 1
        except Exception:
            self.assertFalse(True)

    def test_get(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_get)
        api_client = client.BaseClient(fake_http_client)
        mgr = plan.PlanManager(api_client)
        plan_obj = mgr.get(plan_id='p1')
        self.assert_plan_obj(plan_obj)

    def test_update(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_put)
        api_client = client.BaseClient(fake_http_client)
        mgr = plan.PlanManager(api_client)
        plan_obj = mgr.update('version: 1\nname: ex_plan1\ndescription: dsc1.',
                              plan_id='p1')
        self.assert_plan_obj(plan_obj)



# Copyright (c) 2015 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Tools for interacting with Github.

import base64
import getpass
import json
import random
import re
import string

import httplib2


class GitHubException(Exception):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return "GitHub Exception: %s: %s" % (self.status_code, self.message)


class GitHubAuth(object):
    _github_auth_url = 'https://api.github.com/authorizations'
    _github_repo_hook_url = 'https://api.github.com/repos/%s/hooks'
    _github_user_key_url = 'https://api.github.com/user/keys'
    _github_repo_regex = r'github\.com[:/](.+?)/(.+?)($|/$|\.git$|\.git/$)'

    def __init__(self, git_url, username=None, password=None, repo_token=None):
        self.git_url = git_url

        user_org_name, repo = '', ''

        repo_pat = re.compile(self._github_repo_regex)
        match = repo_pat.search(self.git_url)
        if match:
            user_org_name, repo = match.group(1), match.group(2)
        else:
            raise ValueError("Failed to parse %s." % git_url)

        self.user_org_name = user_org_name

        self.full_repo_name = '/'.join([user_org_name, repo])

        self._repo_token = repo_token
        self._username = username
        self._password = password
        self._otp_required = False

    @property
    def username(self):
        if self._username is None:
            prompt = ("Username for repo '%s' [%s]:" %
                      (self.full_repo_name, self.user_org_name))
            self._username = input(prompt) or self.user_org_name
        return self._username

    @property
    def password(self):
        if self._password is None:
            self._password = getpass.getpass("Password: ")
        return self._password

    @property
    def onetime_password(self):
        # This is prompted for every time it's needed.
        print("Two-Factor Authentication required.")
        return getpass.getpass("2FA Token: ")

    @property
    def repo_token(self):
        if self._repo_token is None:
            self.create_repo_token()
        return self._repo_token

    @property
    def auth_header(self):
        header = {
            'Content-Type': 'application/json',
            }

        # The token on its own should suffice
        if self._repo_token:
            header['Authorization'] = 'token %s' % self._repo_token
            return header

        # This will prompt the user if either name or pass is missing.
        authstring = '%s:%s' % (self.username, self.password)

        basic_auth = base64.encodebytes(bytes(authstring, 'utf-8'))
        basic_auth = basic_auth.decode('utf-8')
        basic_auth = basic_auth.strip()
        header['Authorization'] = 'Basic %s' % basic_auth

        # This will prompt for the OTP.
        if self._otp_required:
            header['x-github-otp'] = self.onetime_password

        return header

    def _send_authed_request(self, url, body_dict):
        body_text = json.dumps(body_dict)

        resp, content = httplib2.Http().request(
            url, 'POST',
            headers=self.auth_header,
            body=body_text)

        if resp.get('status') in ['401']:
            if resp.get('x-github-otp', '').startswith('required'):
                self._otp_required = True
                resp, content = httplib2.Http().request(
                    url, 'POST',
                    headers=self.auth_header,
                    body=body_text)

        return resp, content

    def create_repo_token(self):
        print("Creating repo token")
        note = ''.join(random.sample(string.lowercase, 5))
        auth_info = {
            'scopes': ['repo', 'write:public_key', 'write:repo_hook'],
            'note': 'Solum-status-%s' % note,
            }

        resp, content = self._send_authed_request(
            self._github_auth_url,
            auth_info)

        status_code = int(resp.get('status', '500'))
        response_body = json.loads(content)
        if status_code in [200, 201]:
            self._repo_token = response_body.get('token')
            print("Successfully created repo token %s." % auth_info['note'])
            return self._repo_token
        elif status_code >= 400 and status_code < 600:
            message = response_body.get('message',
                                        'No error message provided.')
            raise GitHubException(status_code, message)

    def create_webhook(self, trigger_uri, workflow=None):
        print("Creating webhook for repo.")
        hook_url = self._github_repo_hook_url % self.full_repo_name
        if workflow is not None:
            # workflow is a list of strings, likely
            # ['unittest', 'build', 'deploy'].
            # They're joined with + and appended to the
            # trigger_uri here.
            wf_query = "?workflow=%s" % '+'.join(workflow)
            trigger_uri += wf_query

        hook_info = {
            'name': 'web',
            'events': ['pull_request', 'commit_comment'],
            'config': {
                'content_type': 'json',
                'url': trigger_uri,
            }
        }
        resp, content = self._send_authed_request(hook_url, hook_info)
        if resp.get('status') in ['200', '201']:
            print("Successfully created webhook.")
        else:
            print("Error creating webhook.")

    def add_ssh_key(self, public_key=None):
        if not public_key:
            print("No public key to upload.")
            return
        print("Uploading public key to user account.")
        api_url = self._github_user_key_url

        key_info = {
            'title': 'devops@Solum',
            'key': public_key,
            }

        resp, content = self._send_authed_request(api_url, key_info)
        if resp.get('status') in ['200', '201']:
            print("Successfully uploaded public key.")
        else:
            print("Error uploading public key.")



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.common.apiclient import client
from solumclient.common.apiclient import fake_client
from solumclient.tests import base
from solumclient.v1 import platform


fixtures = {
    '/v1': {
        'GET': (
            {},
            {
                'uri': 'http://example.com/v1',
                'name': 'solum',
                'type': 'platform',
                'tags': ['solid'],
                'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                'user_id': '55f41cf46df74320b9486a35f5d28a11',
                'description': 'solum native implementation',
                'implementation_version': '2014.1.1',
                'assemblies_uri': 'http://example.com:9777/v1/assemblies',
                'services_uri': 'http://example.com:9777/v1/services',
                'components_uri': 'http://example.com:9777/v1/components',
                'extenstions_uri': 'http://example.com:9777/v1/extenstions'
            }
        ),
    }
}


class PlatformManagerTest(base.TestCase):

    def setUp(self):
        super(PlatformManagerTest, self).setUp()
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures)
        api_client = client.BaseClient(fake_http_client)
        self.mgr = platform.PlatformManager(api_client)

    def test_get(self):
        platform = self.mgr.get()
        self.assertIn('Platform', repr(platform))
        self.assertEqual('http://example.com/v1', platform.uri)
        self.assertEqual('platform', platform.type)
        self.assertEqual('1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                         platform.project_id)
        self.assertEqual('55f41cf46df74320b9486a35f5d28a11', platform.user_id)



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.common.apiclient import base
from solumclient.common.apiclient import client
from solumclient.common.apiclient import fake_client
from solumclient.tests import base as test_base


fixture1 = {
    '/foo_resource': {
        'GET': (
            {},
            {'id': 1, 'name': 'foo'}
        ),
    }
}

fixture2 = {
    '/foo_resource': {
        'GET': (
            {},
            {'foo_resource': {'id': 1, 'name': 'foo'}}
        ),
    }
}

fixture3 = {
    '/foo_resources': {
        'GET': (
            {},
            [
                {'id': 1, 'name': 'foo'},
                {'id': 2, 'name': 'bar'}
            ]
        ),
    }
}

fixture4 = {
    '/foo_resources': {
        'GET': (
            {},
            {'foo_resources': [
                {'id': 1, 'name': 'foo'},
                {'id': 2, 'name': 'bar'}
            ]}
        ),
    }
}


class FooResource(base.Resource):
    pass


class FooResourceManager(base.BaseManager):
    resource_class = FooResource

    def get(self):
        return self._get("/foo_resource")

    def get_with_response_key(self):
        return self._get("/foo_resource", "foo_resource")

    def list(self):
        return self._list("/foo_resources")

    def list_with_response_key(self):
        return self._list("/foo_resources", "foo_resources")


class TestClient(client.BaseClient):

    service_type = "test"

    def __init__(self, http_client, extensions=None):
        super(TestClient, self).__init__(
            http_client, extensions=extensions)

        self.foo_resource = FooResourceManager(self)


class BaseManagerTest(test_base.TestCase):

    def test_get(self):
        http_client = fake_client.FakeHTTPClient(fixtures=fixture1)
        tc = TestClient(http_client)
        tc.foo_resource.get()

    def test_get_with_response_key(self):
        http_client = fake_client.FakeHTTPClient(fixtures=fixture2)
        tc = TestClient(http_client)
        tc.foo_resource.get_with_response_key()

    def test_list(self):
        http_client = fake_client.FakeHTTPClient(fixtures=fixture3)
        tc = TestClient(http_client)
        tc.foo_resource.list()

    def test_list_with_response_key(self):
        http_client = fake_client.FakeHTTPClient(fixtures=fixture4)
        tc = TestClient(http_client)
        tc.foo_resource.list_with_response_key()



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneclient.auth.identity import v2 as v2_auth
from keystoneclient.auth.identity import v3 as v3_auth
from keystoneclient import discover
from keystoneclient import exceptions as ks_exc
from keystoneclient import session
from oslo_utils import strutils
from urllib import parse as urlparse

from solumclient.common.apiclient import auth
from solumclient.common.apiclient import exceptions
from solumclient.common import exc


def _discover_auth_versions(session, auth_url):
    # discover the API versions the server is supporting based on the
    # given URL
    v2_auth_url = None
    v3_auth_url = None
    try:
        ks_discover = discover.Discover(session=session, auth_url=auth_url)
        v2_auth_url = ks_discover.url_for('2.0')
        v3_auth_url = ks_discover.url_for('3.0')
    except ks_exc.DiscoveryFailure:
        raise
    except exceptions.ClientException:
        # Identity service may not support discovery. In that case,
        # try to determine version from auth_url
        url_parts = urlparse.urlparse(auth_url)
        (scheme, netloc, path, params, query, fragment) = url_parts
        path = path.lower()
        if path.startswith('/v3'):
            v3_auth_url = auth_url
        elif path.startswith('/v2'):
            v2_auth_url = auth_url
        else:
            raise exc.CommandError('Unable to determine the Keystone '
                                   'version to authenticate with '
                                   'using the given auth_url.')
    return v2_auth_url, v3_auth_url


def _get_keystone_session(**kwargs):
    # TODO(fabgia): the heavy lifting here should be really done by Keystone.
    # Unfortunately Keystone does not support a richer method to perform
    # discovery and return a single viable URL. A bug against Keystone has
    # been filed: https://bugs.launchpad.net/python-keystoneclient/+bug/1330677

    # first create a Keystone session
    cacert = kwargs.pop('cacert', None)
    cert = kwargs.pop('cert', None)
    key = kwargs.pop('key', None)
    insecure = kwargs.pop('insecure', False)
    auth_url = kwargs.pop('auth_url', None)
    project_id = kwargs.pop('project_id', None)
    project_name = kwargs.pop('project_name', None)

    if insecure:
        verify = False
    else:
        verify = cacert or True

    if cert and key:
        # passing cert and key together is deprecated in favour of the
        # requests lib form of having the cert and key as a tuple
        cert = (cert, key)

    # create the keystone client session
    ks_session = session.Session(verify=verify, cert=cert)
    v2_auth_url, v3_auth_url = _discover_auth_versions(ks_session, auth_url)

    username = kwargs.pop('username', None)
    user_id = kwargs.pop('user_id', None)
    user_domain_name = kwargs.pop('user_domain_name', None)
    user_domain_id = kwargs.pop('user_domain_id', None)
    project_domain_name = kwargs.pop('project_domain_name', None)
    project_domain_id = kwargs.pop('project_domain_id', None)
    auth = None

    use_domain = (user_domain_id or user_domain_name or
                  project_domain_id or project_domain_name)
    use_v3 = v3_auth_url and (use_domain or (not v2_auth_url))
    use_v2 = v2_auth_url and not use_domain

    if use_v3:
        # the auth_url as v3 specified
        # e.g. http://no.where:5000/v3
        # Keystone will return only v3 as viable option
        auth = v3_auth.Password(
            v3_auth_url,
            username=username,
            password=kwargs.pop('password', None),
            user_id=user_id,
            user_domain_name=user_domain_name,
            user_domain_id=user_domain_id,
            project_name=project_name,
            project_id=project_id,
            project_domain_name=project_domain_name,
            project_domain_id=project_domain_id)
    elif use_v2:
        # the auth_url as v2 specified
        # e.g. http://no.where:5000/v2.0
        # Keystone will return only v2 as viable option
        auth = v2_auth.Password(
            v2_auth_url,
            username,
            kwargs.pop('password', None),
            tenant_id=project_id,
            tenant_name=project_name)
    else:
        raise exc.CommandError('Unable to determine the Keystone version '
                               'to authenticate with using the given '
                               'auth_url.')

    ks_session.auth = auth
    return ks_session


def _get_endpoint(ks_session, **kwargs):
    """Get an endpoint using the provided keystone session."""

    # set service specific endpoint types
    endpoint_type = kwargs.get('endpoint_type') or 'publicURL'
    service_type = kwargs.get('service_type') or 'application_deployment'

    endpoint = ks_session.get_endpoint(service_type=service_type,
                                       interface=endpoint_type,
                                       region_name=kwargs.get('region_name'))

    return endpoint


class KeystoneAuthPlugin(auth.BaseAuthPlugin):

    opt_names = ['tenant_id', 'region_name', 'auth_token',
                 'service_type', 'endpoint_type', 'cacert',
                 'auth_url', 'insecure', 'cert_file', 'key_file',
                 'cert', 'key', 'tenant_name', 'project_name',
                 'project_id', 'project_domain_id', 'project_domain_name',
                 'user_id', 'user_domain_id', 'user_domain_name',
                 'password', 'username', 'endpoint']

    def __init__(self, auth_system=None, **kwargs):
        self.opt_names.extend(self.common_opt_names)
        super(KeystoneAuthPlugin, self).__init__(auth_system, **kwargs)

    def _do_authenticate(self, http_client):
        token = self.opts.get('token') or self.opts.get('auth_token')
        endpoint = self.opts.get('endpoint')
        if not (token and endpoint):
            project_id = (self.opts.get('project_id') or
                          self.opts.get('tenant_id'))
            project_name = (self.opts.get('project_name') or
                            self.opts.get('tenant_name'))
            ks_kwargs = {
                'username': self.opts.get('username'),
                'password': self.opts.get('password'),
                'user_id': self.opts.get('user_id'),
                'user_domain_id': self.opts.get('user_domain_id'),
                'user_domain_name': self.opts.get('user_domain_name'),
                'project_id': project_id,
                'project_name': project_name,
                'project_domain_name': self.opts.get('project_domain_name'),
                'project_domain_id': self.opts.get('project_domain_id'),
                'auth_url': self.opts.get('auth_url'),
                'cacert': self.opts.get('cacert'),
                'cert': self.opts.get('cert'),
                'key': self.opts.get('key'),
                'insecure': strutils.bool_from_string(
                    self.opts.get('insecure')),
                'endpoint_type': self.opts.get('endpoint_type'),
            }

            # retrieve session
            ks_session = _get_keystone_session(**ks_kwargs)
            token = ks_session.get_token()
            endpoint = (self.opts.get('endpoint') or
                        _get_endpoint(ks_session, **ks_kwargs))
        self.opts['token'] = token
        self.opts['endpoint'] = endpoint

    def token_and_endpoint(self, endpoint_type, service_type):
        token = self.opts.get('token')
        if callable(token):
            token = token()
        return token, self.opts.get('endpoint')

    def sufficient_options(self):
        """Check if all required options are present.

        :raises: AuthPluginOptionsMissing
        """
        has_token = self.opts.get('token') or self.opts.get('auth_token')
        no_auth = has_token and self.opts.get('endpoint')
        has_project = (self.opts.get('project_id')
                       or (self.opts.get('project_name')
                           and (self.opts.get('user_domain_name')
                           or self.opts.get('user_domain_id'))))
        has_tenant = self.opts.get('tenant_id') or self.opts.get('tenant_name')
        has_credential = (self.opts.get('username')
                          and (has_project or has_tenant)
                          and self.opts.get('password')
                          and self.opts.get('auth_url'))
        missing = not (no_auth or has_credential)
        if missing:
            missing_opts = []
            opts = ['token', 'endpoint', 'username', 'password', 'auth_url',
                    'tenant_id', 'tenant_name']
            for opt in opts:
                if not self.opts.get(opt):
                    missing_opts.append(opt)
            raise exceptions.AuthPluginOptionsMissing(missing_opts)



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.common.apiclient import exceptions
from solumclient.common import exc
from solumclient.tests import base


class FakeResponse(object):
    json_data = {}

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def json(self):
        return self.json_data


class ExceptionTest(base.TestCase):

    def test_from_response_with_status_code_404(self):
        json_data = {"faultstring": "fake message",
                     "debuginfo": "fake details"}
        method = 'GET'
        status_code = 404
        url = 'http://example.com:9777/v1/assemblies/fake-id'
        ex = exc.from_response(
            FakeResponse(status_code=status_code,
                         headers={"Content-Type": "application/json"},
                         json_data=json_data),
            method,
            url
            )
        self.assertIsInstance(ex, exceptions.HttpError)
        self.assertEqual(json_data["faultstring"], ex.message)
        self.assertEqual(json_data["debuginfo"], ex.details)
        self.assertEqual(method, ex.method)
        self.assertEqual(url, ex.url)
        self.assertEqual(status_code, ex.http_status)



# Copyright 2014 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.common.apiclient import base as apiclient_base
from solumclient.common import base as solum_base


class Image(apiclient_base.Resource):
    def __repr__(self):
        return "<Image %s>" % self._info


class ImageManager(solum_base.CrudManager, solum_base.FindMixin):
    resource_class = Image
    collection_key = 'images'
    key = 'image'

    def create(self, **kwargs):
        return super(ImageManager, self).create(base_url="/v1", **kwargs)

    def get(self, **kwargs):
        return super(ImageManager, self).get(base_url="/v1", **kwargs)

    def list(self, **kwargs):
        return super(ImageManager, self).list(base_url="/v1", **kwargs)

    def find(self, **kwargs):
        name_or_uuid = kwargs['name_or_id']
        return super(ImageManager, self).get(base_url="/v1",
                                             image_id=name_or_uuid)



# Copyright 2015 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.common.apiclient import base as apiclient_base
from solumclient.common.apiclient import exceptions
from solumclient.common import base as solum_base
from solumclient.common import exc

from oslo_utils import uuidutils


class App(apiclient_base.Resource):
    def __repr__(self):
        return "<App %s>" % self._info


class AppManager(solum_base.CrudManager, solum_base.FindMixin):
    resource_class = App
    collection_key = 'apps'
    key = 'app'

    def list(self, **kwargs):
        return super(AppManager, self).list(base_url="/v1", **kwargs)

    def create(self, **kwargs):
        return super(AppManager, self).create(base_url="/v1", **kwargs)

    def get(self, **kwargs):
        return super(AppManager, self).get(base_url="/v1", **kwargs)

    def put(self, **kwargs):
        return super(AppManager, self).put(base_url="/v1", **kwargs)

    def patch(self, **kwargs):
        return super(AppManager, self).patch(base_url="/v1", **kwargs)

    def delete(self, **kwargs):
        return super(AppManager, self).delete(base_url="/v1", **kwargs)

    def find(self, **kwargs):
        if 'app_id' in kwargs:
            return super(AppManager, self).get(base_url="/v1", **kwargs)
        elif 'name_or_id' in kwargs:
            name_or_uuid = kwargs['name_or_id']
            try:
                if uuidutils.is_uuid_like(name_or_uuid):
                    return super(AppManager, self).get(
                        base_url="/v1",
                        app_id=name_or_uuid)
                else:
                    return super(AppManager, self).findone(
                        name=name_or_uuid)
            except exceptions.NoUniqueMatch:
                raise exc.NotUnique(resource='App')



# Copyright 2014 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.builder.v1 import image
from solumclient.common.apiclient import client


class Client(client.BaseClient):
    """Client for the Solum v1 API."""

    service_type = "image_builder"

    def __init__(self, http_client, extensions=None):
        """Initialize a new client for the Builder v1 API."""
        super(Client, self).__init__(http_client, extensions)
        self.images = image.ImageManager(self)



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import contextlib
import time

from keystoneclient import adapter
from oslo_utils import strutils

from solumclient.common.apiclient import client as api_client
from solumclient.common import auth
from solumclient.common import exc

API_NAME = 'solum'
VERSION_MAP = {
    '1': 'solumclient.v1.client.Client',
}


def Client(version, *args, **kwargs):
    client_class = api_client.BaseClient.get_class(API_NAME, version,
                                                   VERSION_MAP)
    kwargs['token'] = kwargs.get('token') or kwargs.get('auth_token')
    return client_class(version, *args, **kwargs)


def _adjust_params(kwargs):
    timeout = kwargs.get('timeout')
    if timeout is not None:
        timeout = int(timeout)
        if timeout <= 0:
            timeout = None

    insecure = strutils.bool_from_string(kwargs.get('insecure'))
    verify = kwargs.get('verify')
    if verify is None:
        if insecure:
            verify = False
        else:
            verify = kwargs.get('cacert') or True

    cert = kwargs.get('cert_file')
    key = kwargs.get('key_file')
    if cert and key:
        cert = cert, key
    return {'verify': verify, 'cert': cert, 'timeout': timeout}


def get_client(version, **kwargs):
    """Get an authenticated client, based on the credentials in the kwargs.

    :param api_version: the API version to use ('1')
    :param kwargs: keyword args containing credentials, either:

            * session: a keystoneauth/keystoneclient session object
            * service_type: The default service_type for URL discovery
            * service_name: The default service_name for URL discovery
            * interface: The default interface for URL discovery
                         (Default: public)
            * region_name: The default region_name for URL discovery
            * endpoint_override: Always use this endpoint URL for requests
                                 for this solumclient
            * auth: An auth plugin to use instead of the session one
            * user_agent: The User-Agent string to set
                          (Default is python-solumclient)
            * connect_retries: the maximum number of retries that should be
                               attempted for connection errors
            * logger: A logging object

            or (DEPRECATED):

            * os_token: pre-existing token to re-use
            * os_endpoint: Solum API endpoint

            or (DEPRECATED):

            * os_username: name of user
            * os_password: user's password
            * os_user_id: user's id
            * os_user_domain_id: the domain id of the user
            * os_user_domain_name: the domain name of the user
            * os_project_id: the user project id
            * os_tenant_id: V2 alternative to os_project_id
            * os_project_name: the user project name
            * os_tenant_name: V2 alternative to os_project_name
            * os_project_domain_name: domain name for the user project
            * os_project_domain_id: domain id for the user project
            * os_auth_url: endpoint to authenticate against
            * os_cert|os_cacert: path of CA TLS certificate
            * os_key: SSL private key
            * insecure: allow insecure SSL (no cert verification)
    """
    endpoint = kwargs.get('os_endpoint')

    cli_kwargs = {
        'username': kwargs.get('os_username'),
        'password': kwargs.get('os_password'),
        'tenant_id': (kwargs.get('os_tenant_id')
                      or kwargs.get('os_project_id')),
        'tenant_name': (kwargs.get('os_tenant_name')
                        or kwargs.get('os_project_name')),
        'auth_url': kwargs.get('os_auth_url'),
        'region_name': kwargs.get('os_region_name'),
        'service_type': kwargs.get('os_service_type'),
        'endpoint_type': kwargs.get('os_endpoint_type'),
        'cacert': kwargs.get('os_cacert'),
        'cert_file': kwargs.get('os_cert'),
        'key_file': kwargs.get('os_key'),
        'token': kwargs.get('os_token') or kwargs.get('os_auth_token'),
        'user_domain_name': kwargs.get('os_user_domain_name'),
        'user_domain_id': kwargs.get('os_user_domain_id'),
        'project_domain_name': kwargs.get('os_project_domain_name'),
        'project_domain_id': kwargs.get('os_project_domain_id'),
    }

    cli_kwargs.update(kwargs)
    cli_kwargs.update(_adjust_params(cli_kwargs))

    return Client(version, endpoint, **cli_kwargs)


def get_auth_plugin(**kwargs):
    auth_plugin = auth.KeystoneAuthPlugin(
        auth_url=kwargs.get('auth_url'),
        service_type=kwargs.get('service_type'),
        token=kwargs.get('token'),
        endpoint_type=kwargs.get('endpoint_type'),
        endpoint=kwargs.get('endpoint'),
        cacert=kwargs.get('cacert'),
        tenant_id=kwargs.get('project_id') or kwargs.get('tenant_id'),
        username=kwargs.get('username'),
        password=kwargs.get('password'),
        tenant_name=kwargs.get('tenant_name') or kwargs.get('project_name'),
        user_domain_name=kwargs.get('user_domain_name'),
        user_domain_id=kwargs.get('user_domain_id'),
        project_domain_name=kwargs.get('project_domain_name'),
        project_domain_id=kwargs.get('project_domain_id')
    )
    return auth_plugin


LEGACY_OPTS = ('auth_plugin', 'auth_url', 'token', 'insecure', 'cacert',
               'tenant_id', 'project_id', 'username', 'password',
               'project_name', 'tenant_name',
               'user_domain_name', 'user_domain_id',
               'project_domain_name', 'project_domain_id',
               'key_file', 'cert_file', 'verify', 'timeout', 'cert')


def construct_http_client(**kwargs):
    kwargs = kwargs.copy()
    if kwargs.get('session') is not None:
        # Drop legacy options
        for opt in LEGACY_OPTS:
            kwargs.pop(opt, None)

        service_type_get = kwargs.pop('service_type',
                                      'application_deployment')

        return SessionClient(
            session=kwargs.pop('session'),
            service_type=service_type_get or 'application_deployment',
            interface=kwargs.pop('interface', kwargs.pop('endpoint_type',
                                                         'publicURL')),
            region_name=kwargs.pop('region_name', None),
            user_agent=kwargs.pop('user_agent', 'python-solumclient'),
            auth=kwargs.get('auth', None),
            timings=kwargs.pop('timings', None),
            **kwargs)
    else:
        return api_client.BaseClient(api_client.HTTPClient(
            auth_plugin=kwargs.get('auth_plugin'),
            region_name=kwargs.get('region_name'),
            endpoint_type=kwargs.get('endpoint_type'),
            original_ip=kwargs.get('original_ip'),
            verify=kwargs.get('verify'),
            cert=kwargs.get('cert'),
            timeout=kwargs.get('timeout'),
            timings=kwargs.get('timings'),
            keyring_saver=kwargs.get('keyring_saver'),
            debug=kwargs.get('debug'),
            user_agent=kwargs.get('user_agent'),
            http=kwargs.get('http')
        ))


@contextlib.contextmanager
def record_time(times, enabled, *args):
    """Record the time of a specific action.

    :param times: A list of tuples holds time data.
    :type times: list
    :param enabled: Whether timing is enabled.
    :type enabled: bool
    :param args: Other data to be stored besides time data, these args
                 will be joined to a string.
    """
    if not enabled:
        yield
    else:
        start = time.time()
        yield
        end = time.time()
        times.append((' '.join(args), start, end))


class SessionClient(adapter.LegacyJsonAdapter):
    def __init__(self, *args, **kwargs):
        self.times = []
        self.timings = kwargs.pop('timings', False)
        super(SessionClient, self).__init__(*args, **kwargs)

    def request(self, url, method, **kwargs):
        kwargs.setdefault('headers', kwargs.get('headers', {}))
        raise_exc = kwargs.pop('raise_exc', True)
        with record_time(self.times, self.timings, method, url):
            resp, body = super(SessionClient, self).request(url,
                                                            method,
                                                            raise_exc=False,
                                                            **kwargs)

        if raise_exc and resp.status_code >= 400:
            raise exc.from_response(resp, body)
        return resp



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from urllib import parse as urlparse

from solumclient.common.apiclient import base
from solumclient.common.apiclient import exceptions


class FindMixin(object):
    """Just `findone()`/`findall()` methods.

    Note: this is largely a copy of apiclient.base.ManagerWithFind
          but without the inheritance and the find() method which
          does not clash with the Manager find() - now called findone().
    """

    def findone(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(msg)
        elif num_matches > 1:
            raise exceptions.NoUniqueMatch()
        else:
            return matches[0]

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = kwargs.items()

        for obj in self.list(app_id=kwargs.get('app_id')):
            try:
                if all(getattr(obj, attr) == value
                       for (attr, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue

        return found


class CrudManager(base.CrudManager):
    def list(self, base_url=None, **kwargs):
        """List the collection.

        :param base_url: if provided, the generated URL will be appended to it
        """
        kwargs = self._filter_kwargs(kwargs)

        return self._list(
            '%(base_url)s%(query)s' % {
                'base_url': self.build_url(base_url=base_url, **kwargs),
                'query': '?%s' % urlparse.urlencode(kwargs) if kwargs else '',
            })

    def get(self, **kwargs):
        kwargs = self._filter_kwargs(kwargs)
        return self._get(
            self.build_url(**kwargs))

    def create(self, **kwargs):
        kwargs = self._filter_kwargs(kwargs)
        return self._post(
            self.build_url(**kwargs), kwargs)

    def update(self, **kwargs):
        kwargs = self._filter_kwargs(kwargs)
        params = kwargs.copy()
        params.pop('%s_id' % self.key)

        return self._put(
            self.build_url(**kwargs), params)

    def patch(self, **kwargs):
        kwargs = self._filter_kwargs(kwargs)
        params = kwargs.copy()
        params.pop('%s_id' % self.key)
        params.pop('base_url')

        return self._patch(
            self.build_url(**kwargs), params)



# Copyright (c) 2014 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import fixtures

from solumclient import client as solum_client
from solumclient.common import cli_utils
import solumclient.solum
from solumclient.tests import base


class TestCli_Utils(base.TestCase):
    "Test commandbase"
    scenarios = [
        ('username', dict(
            fake_env={'OS_USERNAME': 'username',
                      'OS_PASSWORD': 'password',
                      'OS_TENANT_NAME': '',
                      'OS_TENANT_ID': '',
                      'OS_PROJECT_NAME': 'project_name',
                      'OS_PROJECT_ID': '',
                      'OS_PROJECT_DOMAIN_NAME': 'default',
                      'OS_PROJECT_DOMAIN_ID': '',
                      'OS_USER_DOMAIN_NAME': 'default',
                      'OS_USER_DOMAIN_ID': '',
                      'OS_REGION_NAME': 'RegionOne',
                      'OS_AUTH_URL': 'http://no.where'},
            output={'solum_api_version': '1',
                    'os_username': 'username',
                    'solum_url': '',
                    'os_tenant_name': '',
                    'os_auth_url': 'http://no.where',
                    'os_password': 'password',
                    'os_tenant_id': '',
                    'os_project_name': 'project_name',
                    'os_project_id': '',
                    'os_project_domain_name': 'default',
                    'os_project_domain_id': '',
                    'os_user_domain_name': 'default',
                    'os_user_domain_id': '',
                    'os_region_name': 'RegionOne',
                    'action': 'create',
                    'json': False,
                    'verify': True,
                    'debug': False
                    })),
        ('token', dict(
            fake_env={'OS_AUTH_TOKEN': '123456',
                      'SOLUM_URL': 'http://10.0.2.15:9777'},
            output={'os_auth_url': '',
                    'solum_url': 'http://10.0.2.15:9777',
                    'solum_api_version': '1',
                    'os_username': '',
                    'os_tenant_name': '',
                    'os_password': '',
                    'os_tenant_id': '',
                    'os_project_name': '',
                    'os_project_id': '',
                    'os_project_domain_name': '',
                    'os_project_domain_id': '',
                    'os_user_domain_name': '',
                    'os_user_domain_id': '',
                    'os_region_name': '',
                    'action': 'create',
                    'json': False,
                    'verify': True,
                    'debug': False
                    })),
        ('solum_url_with_no_token', dict(
            fake_env={'OS_USERNAME': 'username',
                      'OS_PASSWORD': 'password',
                      'OS_TENANT_NAME': '',
                      'OS_TENANT_ID': '',
                      'OS_PROJECT_NAME': 'project_name',
                      'OS_PROJECT_ID': '',
                      'OS_PROJECT_DOMAIN_NAME': 'default',
                      'OS_PROJECT_DOMAIN_ID': '',
                      'OS_USER_DOMAIN_NAME': 'default',
                      'OS_USER_DOMAIN_ID': '',
                      'OS_REGION_NAME': 'RegionOne',
                      'OS_AUTH_URL': 'http://no.where',
                      'SOLUM_URL': 'http://10.0.2.15:9777'},
            output={'os_auth_url': 'http://no.where',
                    'solum_url': 'http://10.0.2.15:9777',
                    'solum_api_version': '1',
                    'os_username': 'username',
                    'os_tenant_name': '',
                    'os_password': 'password',
                    'os_tenant_id': '',
                    'os_project_name': 'project_name',
                    'os_project_id': '',
                    'os_project_domain_name': 'default',
                    'os_project_domain_id': '',
                    'os_user_domain_name': 'default',
                    'os_user_domain_id': '',
                    'os_region_name': 'RegionOne',
                    'action': 'create',
                    'json': False,
                    'verify': True,
                    'debug': False
                    })),
    ]

    # Patch os.environ to avoid reading auth info
    # from environment or command line.
    def make_env(self, exclude=None):
        env = dict((k, v) for k, v in self.fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))

    def fake_argv(self):
        self.useFixture(fixtures.MonkeyPatch('sys.argv', ['foo', 'create']))

    @mock.patch.object(solum_client, "get_client")
    def test_env_parsing(self, mock_get_client):
        parser = solumclient.solum.PermissiveParser()

        self.make_env()
        self.fake_argv()
        FakeCommands(parser)
        mock_get_client.assert_called_once_with(
            self.output['solum_api_version'], **self.output)


class FakeCommands(cli_utils.CommandsBase):
    """Fake command class."""

    def create(self):
        """Fake Create Method."""
        return



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solumclient.common.apiclient import client
from solumclient.common.apiclient import exceptions
from solumclient.common.apiclient import fake_client
from solumclient.tests import base
from solumclient.v1 import component


component_list = [
    {
        'uri': 'http://example.com/v1/components/c1',
        'name': 'php-web-app',
        'type': 'component',
        'description': 'A php web application component',
        'tags': ['web_app'],
        'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
        'user_id': '55f41cf46df74320b9486a35f5d28a11',
        'assembly_link': {
            ' href': 'http://example.com:9777/v1/assembly/a1',
            'target_name': 'a1'},
        'service_links': [{
            'href': 'http://example.com:9777/v1/services/s1',
            'target_name': 's1'}],
        'operations_uri': 'http://example.com:9777/v1/operations/o1',
        'sensors_uri': 'http://example.com:9777/v1/sensors/s1'
    },
    {
        'uri': 'http://example.com/v1/components/c2',
        'name': 'mysql-db',
        'type': 'component',
        'description': 'A mysql db component',
        'tags': ['database'],
        'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
        'user_id': '55f41cf46df74320b9486a35f5d28a11',
        'assembly_link': {
            'href': 'http://example.com:9777/v1/assembly/a2',
            'target_name': 'a2'},
        'service_links': [{
            'href': 'http://example.com:9777/v1/services/s2',
            'target_name': 's2'}],
        'operations_uri': 'http://example.com:9777/v1/operations/o2',
        'sensors_uri': 'http://example.com:9777/v1/sensors/s2'
    }
]

component_fixture = {
    'uri': 'http://example.com/v1/components/c1',
    'name': 'mysql-db',
    'type': 'component',
    'description': 'A mysql db component',
    'tags': ['database'],
    'project_id': '1dae5a09ef2b4d8cbf3594b0eb4f6b94',
    'user_id': '55f41cf46df74320b9486a35f5d28a11',
    'assembly_link': {
        'href': 'http://example.com:9777/v1/assembly/a1',
        'target_name': 'a1'},
    'service_links': [{
        'href': 'http://example.com:9777/v1/services/s1',
        'target_name': 's1'}],
    'operations_uri': 'http://example.com:9777/v1/operations/o1',
    'sensors_uri': 'http://example.com:9777/v1/sensors/o2'
}

fixtures_list = {
    '/v1/components': {
        'GET': (
            {},
            component_list
        ),
    }
}

fixtures_get = {
    '/v1/components/c1': {
        'GET': (
            {},
            component_fixture
        ),
    }
}


fixtures_create = {
    '/v1/components': {
        'POST': (
            {},
            component_fixture
        ),
    }
}

fixtures_put = {
    '/v1/components/c1': {
        'PUT': (
            {},
            component_fixture
        ),
    }
}


class ComponentManagerTest(base.TestCase):

    def test_list_all(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_list)
        api_client = client.BaseClient(fake_http_client)
        self.mgr = component.ComponentManager(api_client)
        components = self.mgr.list()
        self.assertEqual(2, len(components))
        self.assertIn('Component', repr(components[0]))
        self.assertEqual('http://example.com/v1/components/c1',
                         components[0].uri)
        self.assertEqual('http://example.com/v1/components/c2',
                         components[1].uri)

    def test_create(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_create)
        api_client = client.BaseClient(fake_http_client)
        mgr = component.ComponentManager(api_client)
        component_obj = mgr.create()
        self.assertIn('Component', repr(component_obj))
        self.assertEqual('http://example.com/v1/components/c1',
                         component_obj.uri)
        self.assertEqual('component',
                         component_obj.type)
        self.assertEqual('1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                         component_obj.project_id)
        self.assertEqual('55f41cf46df74320b9486a35f5d28a11',
                         component_obj.user_id)

    def test_get(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_get)
        api_client = client.BaseClient(fake_http_client)
        mgr = component.ComponentManager(api_client)
        component_obj = mgr.get(component_id='c1')
        self.assertIn('Component', repr(component_obj))
        self.assertEqual('http://example.com/v1/components/c1',
                         component_obj.uri)
        self.assertEqual('component',
                         component_obj.type)
        self.assertEqual('1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                         component_obj.project_id)
        self.assertEqual('55f41cf46df74320b9486a35f5d28a11',
                         component_obj.user_id)

    def test_put(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_put)
        api_client = client.BaseClient(fake_http_client)
        mgr = component.ComponentManager(api_client)
        component_obj = mgr.put(component_id='c1')
        self.assertIn('Component', repr(component_obj))
        self.assertEqual('http://example.com/v1/components/c1',
                         component_obj.uri)
        self.assertEqual('component',
                         component_obj.type)
        self.assertEqual('1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                         component_obj.project_id)
        self.assertEqual('55f41cf46df74320b9486a35f5d28a11',
                         component_obj.user_id)

    def test_find_one(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_list)
        api_client = client.BaseClient(fake_http_client)
        mgr = component.ComponentManager(api_client)
        components = mgr.findall(name='php-web-app')
        self.assertEqual(1, len(components))
        self.assertIn('Component', repr(components[0]))
        self.assertEqual(component_list[0]['uri'], components[0].uri)

    def test_find_one_only(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_list)
        api_client = client.BaseClient(fake_http_client)
        mgr = component.ComponentManager(api_client)
        result = mgr.find(name_or_id='php-web-app')
        self.assertEqual(component_list[0]['uri'], result.uri)

    def test_find_none(self):
        fake_http_client = fake_client.FakeHTTPClient(fixtures=fixtures_list)
        api_client = client.BaseClient(fake_http_client)
        mgr = component.ComponentManager(api_client)
        self.assertRaises(exceptions.NotFound, mgr.find, name_or_id='test')
